#include <util/delay.h>
#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <string.h>

#define SIZE 20
#define CHAR_WIDTH 5

const char word[SIZE] = "VOTEAZA";
int word_size = strlen(word);
int space_index;

/* matricea literelor:
 * fiecare litera este formata din 5 bytes
 * si astfel cele 8 leduri vor avea asociate un bit
 */
const int letters[][5] = 
{ 
	{0x7e, 0x09, 0x09, 0x09, 0x7e}, //A
	{0x7f, 0x49, 0x49, 0x49, 0x36}, //B
	{0x3e, 0x41, 0x41, 0x41, 0x22}, //C
	{0x7f, 0x41, 0x41, 0x22, 0x1c}, //D
	{0x7f, 0x49, 0x49, 0x49, 0x49}, //E
	{0x7f, 0x09, 0x09, 0x09, 0x09}, //F
	{0x7e, 0x41, 0x49, 0x49, 0x38}, //G
	{0x7f, 0x08, 0x08, 0x08, 0x7f}, //H
	{0x00, 0x41, 0x7f, 0x41, 0x00}, //I
	{0x30, 0x40, 0x41, 0x7f, 0x01}, //J
	{0x7f, 0x08, 0x14, 0x22, 0x41}, //K
	{0x7f, 0x40, 0x40, 0x40, 0x40}, //L
	{0x7f, 0x02, 0x04, 0x02, 0x7f}, //M
	{0x7f, 0x02, 0x0c, 0x10, 0x7f}, //N
	{0x3e, 0x41, 0x41, 0x41, 0x3e}, //O
	{0x7f, 0x49, 0x49, 0x49, 0x36}, //B
	{0x1e, 0x21, 0x21, 0x21, 0x5e}, //Q
	{0x7f, 0x09, 0x19, 0x29, 0x46}, //R
	{0x26, 0x49, 0x49, 0x49, 0x32}, //S
	{0x01, 0x01, 0x7f, 0x01, 0x01}, //T
	{0x3f, 0x40, 0x40, 0x40, 0x3f}, //U
	{0x07, 0x38, 0x40, 0x38, 0x07}, //V
	{0x3f, 0x40, 0x3f, 0x40, 0x3f}, //W
	{0x63, 0x14, 0x08, 0x14, 0x63}, //X
	{0x07, 0x08, 0x70, 0x08, 0x07}, //Y
	{0x61, 0x51, 0x49, 0x45, 0x43}  //Z  
};

volatile unsigned char pov_string[512];
volatile unsigned int pov_string_size = 0;
volatile unsigned int pov_string_i = 0;

/*
 * Adauga spatii dupa litere sau intre cuvinte.
 */
void addSpace(int counter) {
	for (int i = 0; i < counter; i++) {
		pov_string[pov_string_size++] = 0;
	}
}

void createPovString(void)
{
	unsigned int i, j;
	for (i = 0; i < word_size; i++) {
		for (j = 0; j < CHAR_WIDTH ; j++) {
			pov_string[pov_string_size++] = letters[word[i] - 'A'][j];
		}
		addSpace(CHAR_WIDTH);
	}
	
	space_index = word_size * CHAR_WIDTH + 1;
	addSpace(space_index);
}	

/*
 * Intrerupere pentru timer.
 */
ISR(TIMER1_COMPA_vect)
{
	if (pov_string_i < pov_string_size) {
		PORTA = pov_string[pov_string_i++];
	} else {
		pov_string_i = 0;
		_delay_ms(1);
	}
}

void set()
{
	OCR1A = 13;
	/* 
	 * initializez timer1 pentru a masura timpul
	 */
	TCCR1A = 0;
    /* 
	 * mod de functionare in CTC cu TOP la OCR1A, Update - Immediate
	 * prescaler = 1024
	 */
	TCCR1B = (1<<WGM12)| (1 << CS12) | (1 << CS10);
	/* 
	 * activeaza intreruperea de comparare cu OCR1A
	 */
	TIMSK1 |= (1 << OCIE1A);
}

int main()
{
	sei();
	DDRA |= (1 << PA0);
	DDRA |= (1 << PA1);
	DDRA |= (1 << PA2);
	DDRA |= (1 << PA3);
	DDRA |= (1 << PA4);
	DDRA |= (1 << PA5);
	DDRA |= (1 << PA6);
	DDRA |= (1 << PA7);
	createPovString();
	set();
	while (1) {}
	return 0;
}